#################################################################################
#################################################################################
# fixedPLS.R
#
# Contains the fixed function "fregre.pls.cv" of the package "fda.usc", since
# the original one has a bug.
#
# Author: Beatriz Bueno-Larraz
# Date:   05-12-2017
#################################################################################
#################################################################################
fregre.pls.cv_fixed <- function (fdataobj, y, kmax = 8, lambda = 0, P = c(0, 0, 1), 
                   criteria = "SIC", ...) 
{
  if (class(fdataobj) == "fdata.comp") {
    pc <- fdataobj
    fdataobj <- pc$fdataobj
    kmax <- nrow(pc$basis)
  }
  else {
    if (!is.fdata(fdataobj)) 
      fdataobj = fdata(fdataobj)
    omit <- omit.fdata(fdataobj, y)
    fdataobj <- omit[[1]]
    y <- omit[[2]]
  }
  x <- fdataobj[["data"]]
  tt <- fdataobj[["argvals"]]
  rtt <- fdataobj[["rangeval"]]
  n <- nrow(x)
  nc <- ncol(x)
  if (min(lambda) > 0) {
    if (is.vector(P)) {
      P2 <- P.penalty(tt, P = P)
    }
    if (is.logical(lambda[1])) {
      normx <- sqrt(sum(abs((t(x) %*% x)^2)))
      normp <- sqrt(sum(abs(P2^2)))
      normip <- sqrt(sum(abs(diag(nc) + P2)^2))
      pii <- fdata.trace(P2)
      lambda0 <- (-2 * pii + sqrt(4 * pii^2 - 4 * (nc - normx^2) * normp^2))/(2 * normp^2)
      lambda <- seq(0, sqrt(lambda0), len = 10)
    }
  }
  tol <- sqrt(.Machine$double.eps)
  lenrn <- length(lambda)
  ind = 1:kmax
  l = l2 = list()
  ck = 1
  tab = list("AIC", "AICc", "SIC", "SICc", "HQIC", "rho", "CV")
  type.i = pmatch(criteria, tab)
  MSC.min <- Inf
  cv.AIC <- matrix(NA, nrow = lenrn, ncol = kmax)
  if (is.na(type.i)) 
    stop("Error: incorrect criteria")
  else {
    if (type.i < 7) {
      for (r in 1:lenrn) {
        pls <- fdata2pls(fdataobj, y, ncomp = kmax, lambda = lambda[r], P = P, ...)
        for (j in 1:kmax) {
          pls2 <- pls
          pls2$rotation <- pls$rotation[1:j]
          out = fregre.pls(pls2, y, lambda = lambda[r], P = P, ...)
          ck <- out$df
          s2 <- sum(out$residuals^2)/n
          cv.AIC[r, j] <- switch(criteria, AIC = log(s2) + 
                                   2 * (ck)/n, AICc = log(s2) + 2 * (ck)/(n - 
                                                                            ck - 2), SIC = log(s2) + log(n) * ck/n, SICc = log(s2) + 
                                   log(n) * ck/(n - ck - 2), HQIC = log(s2) + 
                                   2 * log(log(n)) * ck/n, rho = {
                                     A <- out$residuals
                                     B <- 1 - diag(out$H)/n
                                     D1 <- (A/B)^2
                                     sum(D1)
                                   })
          if (MSC.min > (cv.AIC[r, j] + tol)) {
            rn.opt <- r
            pc.opt <- j
            MSC.min = cv.AIC[r, j]
          }
        }
      }
    }
    else {
      for (j in 1:kmax) {
        residuals2 <- rep(NA, n)
        for (r in 1:lenrn) {
          for (i in 1:n) {
            out = fregre.pls(fdataobj[-i], y[-i], lambda = lambda[r], 
                             P = P, ...)
            ck <- out$df
            a1 <- out$coefficients[1]
            out$beta.est$data <- matrix(out$beta.est$data, 
                                        nrow = 1)
            b1 <- inprod.fdata(fdata.cen(fdataobj[i], 
                                         out$fdata.comp$mean)[[1]], out$beta.est)
            yp <- a1 + b1
            residuals2[i] <- ((y[i] - yp)/(n - ck))^2
          }
          cv.AIC[r, j] <- sum(residuals2)/n
          if (MSC.min > cv.AIC[r, j]) {
            rn.opt <- r
            pc.opt <- j
            MSC.min = cv.AIC[r, j]
          }
        }
      }
    }
  }
  colnames(cv.AIC) = paste("PLS", 1:kmax, sep = "")
  rownames(cv.AIC) = paste("lambda=", signif(lambda, 4), sep = "")
  
  #fregre = fregre.pls(fdataobj, y, l = 1:pc.opt, lambda = rn.opt, P = P, ...) #WRONG!!
  fregre = fregre.pls(fdataobj, y, l = 1:pc.opt, lambda = lambda[rn.opt], P = P, ...) #FIXED
  MSC.min = cv.AIC[rn.opt, pc.opt]
  return(list(fregre.pls = fregre, pls.opt = 1:pc.opt, lambda.opt = lambda[rn.opt], 
              MSC.min = MSC.min, MSC = cv.AIC))
}