%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Funcion que selecciona hasta dim_red maximos de una curva dada como un
% vector fila. Se consideran h-maximos locales a los mayores maximos
% locales en el intervalo [x-h,x+h]. Se puden calcular simultaneamente para
% multiples h's
%   * Si la funcion tiene mas de dim_red maximos se trunca.
%   * Si la funcion tiene menos de dim_red maximos se devuelven los que
%     haya.
%   * Los maximos se ordenan decrecientemente por valor (curva).
%
% Entrada:
%   - curva: vector fila[1,T] con los valores de la curva en cada nodo.
%   - H: parametro de suavizado. Es el numero de puntos con que comparar a
%        derecha e izquierda. Puede ser un vector para evaluar varios h a
%        la vez.
%   - dim_red: numero maximo de maximos a seleccionar. Si hay mas de
%              dim_red maximos se trunca.
%   - criterio: criterio para ordenar los maximos
%               - 'relevancia' por score (correlacion con la clase)
%               - 'dominio' por amplitud del intervalo donde son maximo
%               global.
%   
% Salida:
%   - nmax: numero de maximos que devuelve la funcion, es el minimo entre
%           dim_red y el numero total de maximos.
%   - S: cell[h,nmax] es el vector con las posiciones de los maximos 
%        (las variables).
%   - score: cell[h,nmax] con curva(S{h}).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function [S, score, nmax]=maxima_selector_new_Par(curva,H,dim_red,criterio)
function maxima_selector_new()
   
   nombre = mfilename();
   mPar = sscanf(nombre,'maxima_selector_Par%d',[1 Inf]);

   fich = fopen(sprintf('curva%d.txt', mPar),'r');
   curva = fscanf(fich, '%f\n');
   fclose(fich);
   curva = curva';
   
   fich = fopen(sprintf('H%d.txt', mPar),'r');
   H = fscanf(fich, '%d\n');
   fclose(fich);
   
   fich = fopen(sprintf('dim_red%d.txt', mPar),'r');
   dim_red = fscanf(fich, '%d');
   fclose(fich);
   
   fich = fopen(sprintf('criterio%d.txt', mPar),'r');
   criterio = fscanf(fich, '%s');
   fclose(fich);
   
    %Reserva de memoria
    S=cell(1,length(H));
    score=cell(1,length(H));
    nmax=zeros(length(H),1);

    %Inicializacion
    %Se a?aden valores minimos en los extremos para poder calcular los
    %maximos en los extremos de la curva
    extra=min(curva)-1;
    x=[extra curva extra];
    extra=extra*ones(1,max(H));  
    ind_maximos=x(2:end-1)>x(1:end-2) & x(2:end-1)>x(3:end); 
    x=[extra curva.*ind_maximos extra];
    maximos=ind_maximos;

    %Se calculan los maximos locales (aquellos que son mayores que sus
    %vecinos inmediatos.


    aux=ones(1,length(curva));
    %x=curva.*maximos;
    %Tiene que ser mayor que el resto de maximos en un radio h
    indice=1;
    a=max(H);
    for i=1:max(H)
       a=a-1;
       %x=[extra x extra];
       %Se compara con desplazarlo a la derecha y a la izquierda.
       aux=aux & x(a+1+i:end-i-a)>x(a+1:end-2*i-a) & x(a+1+i:end-i-a)>x(a+1+2*i:end-a);
       %En cada iteracion quedan aux marca con 1 los puntos que siguen
       %siendo maximos con h=i. Se guardan ordenados por score (relevancia)
       %descendente.
       if(sum(H==i))
           Saux=find(aux); score_aux=curva(Saux);
           [score{indice},b]=sort(score_aux,'descend');       
           S{indice}=Saux(b);
           nmax(indice)=length(Saux);
           %Se trunca si es necesario
           if(strcmp(criterio,'relevancia') && (nmax(indice)>dim_red))
               nmax(indice)=dim_red;
               S{indice}=S{indice}(1:dim_red);
               score{indice}=score{indice}(1:dim_red);
           end
           indice=indice+1;
       end
       if(strcmp(criterio,'dominio'))
           %Acumulamos los puntos que indican el orden por dominio, cuanto
           %mas rondas pase un maximo mas importante es en el criterio dle
           %dominio.
           maximos=maximos+aux;
       end
    end

    if(strcmp(criterio,'dominio'))
        %Continuamos acumulando los puntos de dominio hasta que quede uno
        extra=(min(curva)-1)*ones(1,floor(length(curva)/2));  
        x=[extra curva.*ind_maximos extra];
        a=floor(length(curva)/2)-max(H);
        for i=max(H)+1:floor(length(curva)/2)
            a=a-1;
            aux=aux & x(a+1+i:end-i-a)>x(a+1:end-2*i-a) & x(a+1+i:end-i-a)>x(a+1+2*i:end-a);
            maximos=maximos+aux;
        end
        %Reordenamos los maximos
        for i=1:length(H)
            aux=maximos(S{i});
            [~,indices]=sort(aux,'descend');
            S{i}=S{i}(indices);
            score{i}=score{i}(indices);
            %Se trunca si es necesario
            if(nmax(i)>dim_red)
                nmax(i)=dim_red;
                S{i}=S{i}(1:dim_red);
                score{i}=score{i}(1:dim_red);
            end
        end
    end
    
    save(sprintf('ps%d.mat', mPar),'S')
end
        
  















