################################################################################
#################################################################################
# splitCV.R
#
# Contains the function splitCV
#
# Author: Beatriz Bueno-Larraz
# Date:   22-06-2016
#################################################################################
#################################################################################


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# splitCV
#
# Selects the range of trajectories and responses of an iteration of the k-fold
# cross-validation method for both the train and validation sets.
#
# Input
#   x: Data matrix, one row for each curve.
#   y: Vector of response values.
#   kCV: Total number of folds used for the validation (10 is the standard)
#   k: Current iteration of the validation (i.e. current fold)
#   n: Sample size
#
# Return
#   List containing the following elements:
#     - xE: Data matrix containing the training set for the current fold
#     - xV: Data matrix containing the validation set
#     - yE: Response vector containing the training responses
#     - yV: Response vector containing the validation responses
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
splitCV <- function(x,y,kCV,k,n) {
  nCV = floor(n/kCV) #Size of each fold
  
  # If it is not the last fold
  if (k<kCV) {
    xV = x[((k-1)*nCV+1):(k*nCV),]
    yV = y[((k-1)*nCV+1):(k*nCV)]
    xE = x[(k*nCV+1):n,]
    yE = y[(k*nCV+1):n]
    # If it is the first fold
    if (k>1) {
      xE = rbind(x[1:((k-1)*nCV),], xE)
      yE = c(y[1:((k-1)*nCV)], yE)
    }
  } else {
  # If it is the last fold
    xV = x[((k-1)*nCV+1):n,]
    yV = y[((k-1)*nCV+1):n]
    xE = x[1:((k-1)*nCV),]
    yE = y[1:((k-1)*nCV)]
  }
  
  return(list(xV=xV,xE=xE,yV=yV,yE=yE))
}
